from argparse import Namespace

from yamapan import subcommand
from yamapan.config import (
    CONFIG_FILENAME,
    CONFIG_TEMPLATE,
    GITIGNORE_TEMPLATE,
)


@subcommand("init")
def init(namespace: Namespace, args: list[str]) -> int:
    with open(CONFIG_FILENAME, "w") as f:
        f.write(CONFIG_TEMPLATE)
    with open(".gitignore", "w") as f:
        f.write(GITIGNORE_TEMPLATE)
    return 0



from argparse import Namespace

from yamapan import subcommand
from yamapan.util import get_workspace_path, run_command


@subcommand("build")
def build(namespace: Namespace, args: list[str]) -> int:
    workspace_path = get_workspace_path()
    return run_command(
        f"colcon build {'' if namespace.no_symlink else '--symlink-install'} {' '.join(args)}",
        ros_env=True,
        cwd=workspace_path,
    )


build.add_argument(
    "--no-symlink",
    action="store_true",
    help="do not use --symlink-install",
)



from argparse import Namespace

import yaml

from yamapan import subcommand
from yamapan.util import get_workspace_path, load_config, run_command


@subcommand("resolve")
def resolve(namespace: Namespace, args: list[str]) -> int:
    workspace_path = get_workspace_path()
    config = load_config()
    ret = run_command(
        "vcs import",
        cwd=workspace_path,
        input=yaml.dump({"repositories": config.repositories}).encode(),
    )
    if ret != 0:
        return ret
    ret = run_command("rosdep update")
    if ret != 0:
        return ret
    return run_command(
        "rosdep install -y --from-paths . --ignore-src --rosdistro $ROS_DISTRO",
        ros_env=True,
        cwd=workspace_path,
    )



from pathlib import Path
from typing import Any

from pydantic import BaseModel

CONFIG_FILENAME = "yamapan.toml"
CONFIG_TEMPLATE = """[aliases]
# build = "yamapan build --cmake-args -DCMAKE_C_COMPILER_LAUNCHER=ccache -DCMAKE_CXX_COMPILER_LAUNCHER=ccache"

[repositories]
# "ext/voicevox_ros2" = { type = "git", url = "https://github.com/tutrobo/voicevox_ros2.git", version = "master" }
"""

GLOBAL_CONFIG_FILENAME = ".yamapan.toml"

GITIGNORE_TEMPLATE = """build/
install/
log/
"""


class Config(BaseModel):
    aliases: dict[str, str]
    repositories: dict[str, Any]


class GlobalConfig(BaseModel):
    ros_install_prefix: Path



from argparse import ArgumentParser, Namespace
from typing import Callable

parser = ArgumentParser(
    prog="yamapan",
    description="Yet Another MetA worksPAce maNager for ROS 2",
)
parser.add_argument("-v", "--version", action="version", version="%(prog)s 0.0.5")
subparsers = parser.add_subparsers()


def subcommand(name: str):
    def _subcommand(handler: Callable[[Namespace, list[str]], int]):
        parser = subparsers.add_parser(name)
        parser.set_defaults(handler=handler)
        return parser

    return _subcommand


def main() -> int:
    namespace, args = parser.parse_known_args()
    if hasattr(namespace, "handler"):
        return namespace.handler(namespace, args)
    else:
        parser.print_help()
        return 1


import yamapan.alias as _
import yamapan.build as _
import yamapan.clean as _
import yamapan.init as _
import yamapan.resolve as _
import yamapan.ros2 as _
import yamapan.rqt as _



import shutil
from argparse import Namespace

from yamapan import subcommand
from yamapan.util import get_workspace_path


@subcommand("clean")
def clean(namespace: Namespace, args: list[str]) -> int:
    workspace_path = get_workspace_path()
    shutil.rmtree(workspace_path / "build", ignore_errors=True)
    shutil.rmtree(workspace_path / "install", ignore_errors=True)
    shutil.rmtree(workspace_path / "log", ignore_errors=True)
    return 0



from argparse import Namespace

from yamapan import subcommand
from yamapan.util import run_command


@subcommand("ros2")
def ros2(namespace: Namespace, args: list[str]) -> int:
    return run_command(
        f"ros2 {' '.join(args)}",
        ros_env=True,
    )



import subprocess
import sys
from pathlib import Path

import tomli

from yamapan.config import CONFIG_FILENAME, GLOBAL_CONFIG_FILENAME, Config, GlobalConfig


def get_workspace_path() -> Path:
    def _get_workspace_path(path: Path) -> Path | None:
        config_path = path / CONFIG_FILENAME
        if config_path.is_file():
            return path
        elif path.parent == path:
            return None
        return _get_workspace_path(path.parent)

    workspace_path = _get_workspace_path(Path.cwd())
    if workspace_path is None:
        sys.exit("yamapan: not in a workspace")
    return workspace_path


def load_config() -> Config:
    with open(get_workspace_path() / CONFIG_FILENAME, "rb") as f:
        return Config.model_validate(tomli.load(f))


def load_global_config() -> GlobalConfig:
    try:
        with open(Path.home() / GLOBAL_CONFIG_FILENAME, "rb") as f:
            return GlobalConfig.model_validate(tomli.load(f))
    except:
        sys.exit(f"""yamapan: "~/{GLOBAL_CONFIG_FILENAME}" not found
Please run the following command:

echo "ros_install_prefix = \\"/opt/ros/$ROS_DISTRO\\"" > ~/{GLOBAL_CONFIG_FILENAME}""")


def get_setup_sh_path() -> Path:
    setup_sh = get_workspace_path() / "install" / "setup.sh"
    if setup_sh.is_file():
        return setup_sh
    global_config = load_global_config()
    return global_config.ros_install_prefix / "setup.sh"


def run_command(
    command: str,
    ros_env: bool = False,
    cwd: Path | None = None,
    input: bytes | None = None,
) -> int:
    with subprocess.Popen(
        f"{f'. {get_setup_sh_path()} && ' if ros_env else ''}{command}",
        stdin=None if input is None else subprocess.PIPE,
        shell=True,
        cwd=cwd,
    ) as process:
        try:
            process.communicate(input)
        except KeyboardInterrupt:
            process.send_signal(subprocess.signal.SIGINT)
            process.wait()
        except:
            process.kill()
            raise
        return process.poll()



import sys
from argparse import Namespace

from yamapan import subcommand
from yamapan.util import get_workspace_path, load_config, run_command


@subcommand("alias")
def alias(namespace: Namespace, args: list[str]) -> int:
    workspace_path = get_workspace_path()
    config = load_config()
    alias = config.aliases.get(namespace.name)
    if alias is None:
        sys.exit(f"yamapan: alias not found: '{namespace.name}'")
    return run_command(
        f"{alias} {' '.join(args)}",
        cwd=workspace_path,
    )


alias.add_argument("name", help="alias name")
